/*
** File: admxrc6tl.h  
** Project: ADM-XRC core driver
** Purpose: Definitions specific to for ADM-XRC-6TL.
**
** (C) Copyright Alpha Data 2009-2010
*/

#ifndef _ADATA_CORE_ADMXRC6TL_H
#define _ADATA_CORE_ADMXRC6TL_H

#include <df.h>
#include "adb3.h"

#define NUM_ADB3_ADMXRC6TL_DMA_CHANNEL (1)

/* Always LITTLE ENDIAN in hardware */
#pragma pack(1)
typedef struct _ModelRegsAdmxrc6tl {
  uint32_t pcbRev;             /* 0x0000 */
	uint32_t pwrStat;            /* 0x0004 */
	uint32_t _reserved1[2];      /* 0x0008 */
	uint32_t lclkCtl;            /* 0x0010 */
	uint32_t lclkStat1;          /* 0x0014 */
	uint32_t lclkStat2;          /* 0x0018 */
	uint32_t lclkStat3;          /* 0x001C */
	uint8_t  selectMap[4];       /* 0x0020 */
	uint32_t fpgaCtl;            /* 0x0024 */
  uint32_t fpgaRead;           /* 0x0028 */
  uint32_t _reserved2;         /* 0x002C */
	uint32_t intCtl;             /* 0x0030 */
	uint32_t intStat;            /* 0x0034 */
	uint32_t _reserved3[2];      /* 0x0038 */
	uint32_t flashCtl;           /* 0x0040 */
	uint32_t flashAddr;          /* 0x0044 */
	uint32_t flashData;          /* 0x0048 */
	uint32_t _reserved4;         /* 0x004C */
  uint32_t gpioBToT;           /* 0x0050 */
  uint32_t gpioTToT;           /* 0x0054 */
  uint32_t gtpStat;            /* 0x0058 */
	uint32_t _reserved5;         /* 0x005C */
  uint32_t mptlRecovery;       /* 0x0060 */
  uint32_t mptlStat;           /* 0x0064 */
  uint32_t mptlError;          /* 0x0068 */
	uint32_t _reserved6[229];    /* 0x006C */
  uint32_t i2cCtl;             /* 0x0400 */
  uint32_t i2cStat;            /* 0x0404 */
  uint32_t sysMonStat;         /* 0x0408 */
  uint32_t xrmStat;            /* 0x040C */
	uint32_t _reserved7[60];     /* 0x0410 */
  uint8_t  sysMonBuf[256];     /* 0x0500 */
  uint8_t  xrmBuf[256];        /* 0x0600 */
  uint8_t  i2cBuf[256];        /* 0x0700 */
} ModelRegsAdmxrc6tl;          /* 0x0800 */
#pragma pack()

/* Fields for ModelRegsAdmxrc6tl::intCtl */
#define ADMXRC6TL_INTCTL_FP_ENABLE   (0x1U << 0U)
#define ADMXRC6TL_INTCTL_SYS_ENABLE  (0x1U << 2U)
#define ADMXRC6TL_INTCTL_FP_LEVEL    (0x1U << 8U)
#define ADMXRC6TL_INTCTL_SYS_LEVEL   (0x1U << 10U)
#define ADMXRC6TL_INTCTL_ENABLE_BITS (ADMXRC6TL_INTCTL_FP_ENABLE | ADMXRC6TL_INTCTL_SYS_ENABLE)

/* Fields for ModelRegsAdmxrc6tl::intStat */
#define ADMXRC6TL_INTSTAT_FP         (0x1U << 0U)
#define ADMXRC6TL_INTSTAT_SYS        (0x1U << 2U)

/* Always LITTLE ENDIAN when stored in hardware */
#pragma pack(1)
typedef struct _VpdAdmxrc6tlRev0 {
	uint16_t version;       /* Revision of this structure (0) */
	uint16_t length;        /* Length of this structure including checksum */
	uint32_t serial;
	uint32_t modifications;
	uint32_t lclkRef;
	struct {
		uint16_t speed;       /* In units of 100 kHz */
		uint8_t	 order;       /* Log base 2 of size in words */
    uint8_t  _reserved1;
    uint16_t link;        /* Byte offset in VPD of extended info */
    uint16_t _reserved2;
	} sdram[4];
  struct {
    uint8_t  stepping[8];
    uint8_t  speedGrade[4];
    uint16_t device;
    uint8_t  tempGrade;
    uint8_t  _reserved1;
  } fpga;
	uint8_t  pcbRev;
	uint8_t  cpldRev;
  uint8_t  _reserved1;
  uint8_t  checksum;      /* This MUST ALWAYS be the last member in the struct */
} VpdAdmxrc6tlRev0;
#pragma pack()

/* Values for VpdAdmxrc6tlRev0::modifications */
#define ADMXRC6TL_VPD_MOD_NOTFITTED_XMC_P16 (0x1U << 0U) /* XMC P16 connector NOT fitted */
#define ADMXRC6TL_VPD_MOD_NOTFITTED_XMC_PN4 (0x1U << 1U) /* PMC Pn4 connector NOT fitted */

/* Values for VpdAdmxrc6tlRev0::fpga::tempGrade */
#define ADMXRC6TL_VPD_TEMPGRADE_COMMERCIAL  (0)
#define ADMXRC6TL_VPD_TEMPGRADE_INDUSTRIAL  (1)

#endif
