/*
** File: corevpd.c  
** Project: ADB3 core driver
** Purpose: Implements functions for VPD read and write in Core Interface.
**
** (C) Copyright Alpha Data 2009-2010
*/

#include <df.h>

#include "corenotify.h"

boolean_t
coreNotificationReg(
  void* pInterfaceContext,
  unsigned int notifyType,
  CoreNotification* pNotification,
  CoreNotificationCallback* pCallback,
  void* pCallbackContext)
{
  Adb3CoreDeviceContext* pDevCtx = (Adb3CoreDeviceContext*)pInterfaceContext;
  unsigned int index;
  DfSpinLockFlags f;

  pNotification->system.notifyType = notifyType;
  pNotification->system.pNext = NULL;
  pNotification->system.pCallback = pCallback;
  pNotification->system.pCallbackContext = pCallbackContext;
  if (notifyType >= CORE_NOTIFY_FPGA_INTERRUPT(0) && notifyType < CORE_NOTIFY_FPGA_INTERRUPT(pDevCtx->info.bootstrap.numTargetFpga)) {
    index = notifyType - CORE_NOTIFY_FPGA_INTERRUPT(0);
    /* Add this node to the linked list */
    f = dfSpinLockGet(&pDevCtx->notification.fpgaInterrupt[index].lock);
    pNotification->system.pPrev = pDevCtx->notification.fpgaInterrupt[index].pTail;
    if (NULL == pDevCtx->notification.fpgaInterrupt[index].pTail) {
      pNotification->system.pPrev = pDevCtx->notification.fpgaInterrupt[index].pHead = pNotification;
      pNotification->system.pPrev = pDevCtx->notification.fpgaInterrupt[index].pTail = pNotification;
    } else {
      pDevCtx->notification.fpgaInterrupt[index].pTail->system.pNext = pNotification;
      pDevCtx->notification.fpgaInterrupt[index].pTail = pNotification;
    }
    dfSpinLockPut(&pDevCtx->notification.fpgaInterrupt[index].lock, f);
  } else {
    /* Invalid notification type */
    return FALSE;
  }

  return TRUE;
}

boolean_t
coreNotificationUnreg(
  void* pInterfaceContext,
  CoreNotification* pNotification)
{
  Adb3CoreDeviceContext* pDevCtx = (Adb3CoreDeviceContext*)pInterfaceContext;
  unsigned int notifyType;
  unsigned int index;
  DfSpinLockFlags f;

  notifyType = pNotification->system.notifyType;
  if (notifyType >= CORE_NOTIFY_FPGA_INTERRUPT(0) && notifyType < CORE_NOTIFY_FPGA_INTERRUPT(pDevCtx->info.bootstrap.numTargetFpga)) {
    index = notifyType - CORE_NOTIFY_FPGA_INTERRUPT(0);
    /* Remove this node from the linked list */
    f = dfSpinLockGet(&pDevCtx->notification.fpgaInterrupt[index].lock);
    if (NULL == pNotification->system.pPrev) {
      pDevCtx->notification.fpgaInterrupt[index].pHead = pNotification->system.pNext;
    } else {
      pNotification->system.pPrev->system.pNext = pNotification->system.pNext;
    }
    if (NULL == pNotification->system.pNext) {
      pDevCtx->notification.fpgaInterrupt[index].pTail = pNotification->system.pPrev;
    } else {
      pNotification->system.pNext->system.pPrev = pNotification->system.pPrev;
    }
    dfSpinLockPut(&pDevCtx->notification.fpgaInterrupt[index].lock, f);
  } else {
    /* Invalid notification type */
    return FALSE;
  }

  return TRUE;
}
