/*
** File: mapio.c  
** Project: Linux Driver Framework
** Purpose: Mechanism by which OS-independent driver code can map device memory
**          into the driver's address space.
**
** (C) Copyright Alpha Data 2010
*/

#include "df.h"
#include "dflinux.h"

void*
dfMapIoSpace(
  DfDeviceObject* pDevObj, /* not used */
  uint64_t systemAddress,
  uint64_t length)
{
  /* Sanity check */
  if (sizeof(uint64_t) > sizeof(resource_size_t) && systemAddress > (resource_size_t)-1) {
    return NULL;
  }
  /* Sanity check */
  if (length > (unsigned long)-1) {
    return NULL;
  }
  return ioremap((resource_size_t)systemAddress, (unsigned long)length);
}

boolean_t
dfUnmapIoSpace(
  DfDeviceObject* pDevObj, /* not used */
  void* pKernelAddress)
{
  iounmap(pKernelAddress);
  return TRUE;
}
