/*
** File: pool.c  
** Project: Linux Driver Framework
** Purpose: Pool creation and destruction functions
**
** (C) Copyright Alpha Data 2010
*/

#include "df.h"
#include "dflinux.h"

boolean_t
_dfPoolInit(
  DfPool* pPool,
  size_t itemSize)
{
  /* Ensure that name of cache is unique */
  snprintf(pPool->system.name, DFPOOL_NAME_MAX_LENGTH, "%p:%s", pPool, g_dfDriverName);
  pPool->system.name[DFPOOL_NAME_MAX_LENGTH - 1] = '\0';
#if KMEM_CACHE_CREATE_5
  pPool->system.pCache = kmem_cache_create(pPool->system.name, itemSize, 0, SLAB_HWCACHE_ALIGN, NULL);
#else
  pPool->system.pCache = kmem_cache_create(pPool->system.name, itemSize, 0, SLAB_HWCACHE_ALIGN, NULL, NULL);
#endif
  if (NULL == pPool->system.pCache) {
    return FALSE;
  }
#if DF_DBG_BUILD
  pPool->system.itemSize = itemSize;
  atomic_set(&pPool->system.count, 0);
#endif

  return TRUE;
}

void
dfPoolUninit(
  DfPool* pPool)
{
#if DF_DBG_BUILD
  dfAssert(0 == atomic_read(&pPool->system.count));
  dfAssert(NULL != pPool->system.pCache);
#endif
  if (NULL != pPool->system.pCache) {
    kmem_cache_destroy(pPool->system.pCache);
    pPool->system.pCache = NULL;
  }
}
