/*
** File: main.c  
** Project: ADB3 monolithic driver
** Purpose: OS-independent entry point into ADB3 monolithic driver
**
** (C) Copyright Alpha Data 2010
*/

#include <df.h>

#include "device.h"
#include <core/main.h>
#include <modules/admxrc3/main.h>

const char g_dfDriverName[] = "adb3";

static boolean_t
onAdd(
  DfDeviceObject* pDevObj)
{
  DeviceContext* pDevCtx;
  boolean_t bSuccess;
  
  dfDebugPrint(1, ("onAdd: entered, pDevObj=0x%p\n", pDevObj));
  
  /* Initialize the device context */
  pDevCtx = (DeviceContext*)dfDeviceGetContext(pDevObj);
  pDevCtx->pDevObj = pDevObj;
  
  bSuccess = adb3CoreOnAdd(pDevObj, &pDevCtx->adb3core);
  if (!bSuccess) {
    return FALSE;
  }
  bSuccess = admxrc3OnAdd(pDevObj, &pDevCtx->admxrc3);
  if (!bSuccess) {
    adb3CoreOnRemove(pDevObj, &pDevCtx->adb3core);
    return FALSE;
  }
  return TRUE;
}

static void
onRemove(
  DfDeviceObject* pDevObj)
{
  DeviceContext* pDevCtx;

  dfDebugPrint(1, ("onRemove: entered, pDevObj=0x%p\n", pDevObj));
  
  pDevCtx = (DeviceContext*)dfDeviceGetContext(pDevObj);
  admxrc3OnRemove(pDevObj, &pDevCtx->admxrc3);
  adb3CoreOnRemove(pDevObj, &pDevCtx->adb3core);
}

static boolean_t
onStart(
  DfDeviceObject* pDevObj,
  DfBusResources* pRawRes,
  DfBusResources* pTranslatedRes)
{
  DeviceContext* pDevCtx;
  boolean_t bSuccess;
  
  dfDebugPrint(1, ("onStart: entered, pDevObj=0x%p\n", pDevObj));

  pDevCtx = (DeviceContext*)dfDeviceGetContext(pDevObj);
  bSuccess = adb3CoreOnStart(pDevObj, &pDevCtx->adb3core, pRawRes, pTranslatedRes);
  if (!bSuccess) {
    return FALSE;
  }
  bSuccess = admxrc3OnStart(pDevObj, &pDevCtx->admxrc3, pRawRes, pTranslatedRes);
  if (!bSuccess) {
    adb3CoreOnStop(pDevObj, &pDevCtx->adb3core);
    return FALSE;
  }
  return TRUE;
}

static void
onStop(
  DfDeviceObject* pDevObj)
{
  DeviceContext* pDevCtx;

  dfDebugPrint(1, ("onStop: entered, pDevObj=0x%p\n", pDevObj));

  pDevCtx = (DeviceContext*)dfDeviceGetContext(pDevObj);
  admxrc3OnStop(pDevObj, &pDevCtx->admxrc3);
  adb3CoreOnStop(pDevObj, &pDevCtx->adb3core);
}

boolean_t
dfDriverEntry(
  DfDriverObject* pDrvObj)
{
  boolean_t bSuccess;

  dfDebugPrint(0, ("dfDriverEntry: ADB3 Monolithic Driver, version=%lu.%lu.%lu.%lu\n",
    (unsigned long)ADB3_VERSION_0, (unsigned long)ADB3_VERSION_1, (unsigned long)ADB3_VERSION_2, (unsigned long)ADB3_VERSION_3));

  pDrvObj->deviceContextSize = sizeof(Adb3CoreDeviceContext) + sizeof(Admxrc3DeviceContext);
  pDrvObj->methods.pOnAdd = onAdd;
  pDrvObj->methods.pOnRemove = onRemove;
  pDrvObj->methods.pOnStart = onStart;
  pDrvObj->methods.pOnStop = onStop;

  bSuccess = adb3CoreDriverEntry(pDrvObj);
  if (!bSuccess) {
    return FALSE;
  }
  bSuccess = admxrc3DriverEntry(pDrvObj);
  if (!bSuccess) {
    adb3CoreDriverExit(pDrvObj);
    return FALSE;
  }
  return TRUE;
}

extern void
dfDriverExit(
  DfDriverObject* pDrvObj)
{
  dfDebugPrint(0, ("dfDriverExit: ADB3 Monolithic Driver exiting\n"));
  admxrc3DriverExit(pDrvObj);
  adb3CoreDriverExit(pDrvObj);
}
