#ifndef _ADATA_ADMXRC3_STRUCTS_H
#define _ADATA_ADMXRC3_STRUCTS_H

/*
** structs.h - structs used in ADMXRC3 API
**
** (C) Copyright Alpha Data 2009-2010
**
** The integer datatypes defined below prefixed by "_ADMXRC3_", such as
** "_ADMXRC3_UINT64" are defined in order to increase the portability of
** this header file but should NOT be used by application code that
** makes use of the ADMXRC3 API.
*/

#if defined(_WIN32) || defined(_MSC_VER)

/* Windows */

typedef UINT64 _ADMXRC3_UINT64;
typedef UINT32 _ADMXRC3_UINT32;
typedef UINT16 _ADMXRC3_UINT16;
typedef UINT8  _ADMXRC3_UINT8;
typedef UINT8  _ADMXRC3_BYTE;
typedef BOOL   _ADMXRC3_BOOL;

#elif defined(__linux) || defined(__VXWORKS__) || defined(__vxworks)

/* Linux or VxWorks - assume C99-compliant */

#include <stdint.h>

typedef uint64_t _ADMXRC3_UINT64;
typedef uint32_t _ADMXRC3_UINT32;
typedef uint16_t _ADMXRC3_UINT16;
typedef uint8_t  _ADMXRC3_UINT8;
typedef uint8_t  _ADMXRC3_BYTE;
typedef int      _ADMXRC3_BOOL;

#else

# error Cannot detect target operating system.

#endif

/*
** Bank information structure, returned by ADMXRC3_GetBankInfo
*/

typedef struct _ADMXRC3_BANK_INFO {
  _ADMXRC3_UINT64 MaximumFrequency;  /* Maximum operating frequency, in Hz                */
  _ADMXRC3_UINT64 MinimumFrequency;  /* Minimum operating frequency, in Hz                */
  _ADMXRC3_UINT64 PhysicalSize;      /* Number of physical data words                     */
  _ADMXRC3_UINT16 PhysicalDataWidth; /* Number of physical data lines                     */
  _ADMXRC3_UINT16 PhysicalECCWidth;  /* Number of physical ECC lines                      */
  _ADMXRC3_UINT16 PhysicalWidth;     /* Number of physical data & ECC lines               */
  _ADMXRC3_UINT16 Reserved1;
  _ADMXRC3_UINT32 TypeMask;          /* Bitmask of RAM mode(s) supported                  */
  _ADMXRC3_UINT32 ConnectivityMask;  /* Bitmask of target FPGAs that bank is connected to */
  _ADMXRC3_BOOL   Present;           /* TRUE if physically present                        */
} ADMXRC3_BANK_INFO;

/*
** Notes:
**  1. If MaximimFrequency == 0, then both MaximimFrequency & MininimFrequency must
**     be considered unknown.
*/

/*
** Bitstream structure, returned by ADMXRC3_LoadBitstream
*/

/* ANSI string version */
typedef struct _ADMXRC3_BITSTREAMA {
  char            Identifier[32];  /* Same format as in ADMXRC3_FPGA_INFOA */
  _ADMXRC3_UINT32 Length;          /* Length in bytes of 'Data' field */
  _ADMXRC3_BYTE   Data[4];         /* Variable length array containing SelectMap data (4 bytes to avoid struct packing issues) */
} ADMXRC3_BITSTREAMA;

/* Unicode string version */
typedef struct _ADMXRC3_BITSTREAMW {
  wchar_t         Identifier[32]; /* Same format as in ADMXRC3_FPGA_INFOW */
  _ADMXRC3_UINT32 Length;         /* Length in bytes of 'Data' field */
  _ADMXRC3_BYTE   Data[4];        /* Variable length array containing SelectMap data (4 bytes to avoid struct packing issues) */
} ADMXRC3_BITSTREAMW;

#if defined(_UNICODE)
# define ADMXRC3_BITSTREAM ADMXRC3_BITSTREAMW
#else
# define ADMXRC3_BITSTREAM ADMXRC3_BITSTREAMA
#endif

/*
** Card information structure, returned by ADMXRC3_GetCardInfo
*/

typedef struct _ADMXRC3_CARD_INFO {
  _ADMXRC3_UINT32    SerialNumber;
  _ADMXRC3_UINT32    Reserved1;
  ADMXRC3_MODEL_TYPE Model;
  unsigned int       NumClockGen;
  unsigned int       NumDmaChannel;
  unsigned int       NumFlashBank;
  unsigned int       NumMemoryBank;
  unsigned int       NumTargetFpga;
  unsigned int       NumWindow;
  _ADMXRC3_UINT32    MemoryBankPresent;
} ADMXRC3_CARD_INFO;

/*
** Flash information structure, returned by ADMXRC3_GetFlashInfo
*/

typedef struct _ADMXRC3_FLASH_INFOA {
  char            Identifier[32];  /* Freeform string                                           */
  _ADMXRC3_UINT64 Size;            /* Size of Flash, in bytes                                   */
  _ADMXRC3_UINT64 UseableStart;    /* Address (in bytes) of beginning of user-programmable area */
  _ADMXRC3_UINT64 UseableLength;   /* Length (in bytes) of user-programmable area               */
} ADMXRC3_FLASH_INFOA;

typedef struct _ADMXRC3_FLASH_INFOW {
  wchar_t         Identifier[32];  /* Freeform string                                           */
  _ADMXRC3_UINT64 Size;            /* Size of Flash, in bytes                                   */
  _ADMXRC3_UINT64 UseableStart;    /* Address (in bytes) of beginning of user-programmable area */
  _ADMXRC3_UINT64 UseableLength;   /* Length (in bytes) of user-programmable area               */
} ADMXRC3_FLASH_INFOW;

#if defined(_UNICODE)
#   define ADMXRC3_FLASH_INFO ADMXRC3_FLASH_INFOW
#else
#   define ADMXRC3_FLASH_INFO ADMXRC3_FLASH_INFOA
#endif

/*
** Flash block information structure, returned by ADMXRC3_GetFlashBlockInfo
*/

typedef struct _ADMXRC3_FLASHBLOCK_INFO {
  _ADMXRC3_UINT64 Address;        /* Address (in bytes) of Flash block                         */
  _ADMXRC3_UINT64 Length;         /* Length (in bytes) of Flash block                          */
  _ADMXRC3_UINT32 Flags;          /* See flag definitions below                                */
} ADMXRC3_FLASHBLOCK_INFO;

/*
** FPGA information structure, returned by ADMXRC3_GetFpgaInfo
*/

#if defined(_UNICODE)
#   define ADMXRC3_FPGA_INFO ADMXRC3_FPGA_INFOW
#else
#   define ADMXRC3_FPGA_INFO ADMXRC3_FPGA_INFOA
#endif

/* ANSI string version */
typedef struct _ADMXRC3_FPGA_INFOA {
  char                   Identifier[32];    /* Upper case string in form as seen in .BIT file          */
  ADMXRC3_FPGA_TYPE      DeviceCode;        /* Device; see ADMXRC2_FPGA_TYPE                           */
  ADMXRC3_FAMILY_TYPE    FamilyCode;        /* Family; see ADMXRC2_FAMILY_TYPE                         */
  ADMXRC3_SUBFAMILY_TYPE SubfamilyCode;     /* Subfamily; see ADMXRC2_SUBFAMILY_TYPE                   */
  ADMXRC3_PACKAGE_TYPE   PackageCode;       /* Package; see ADMXRC2_PACKAGE_TYPE                       */
  char                   Device[16];        /* As 'DeviceCode', but upper case string e.g. "2VP7"      */
  char                   Package[16];       /* As 'Package', but upper case string e.g. "FG456"        */
  _ADMXRC3_UINT32        Flags;             /* Indicates which of following fields are valid           */
  char                   SpeedGrade[8];     /* Speed grade including "C" or "I"                        */
  char                   Stepping[8];       /* Stepping level, or -1 if not known                      */
  _ADMXRC3_BOOL          Present;           /* TRUE if the device is physically present                */ 
} ADMXRC3_FPGA_INFOA;

/* Unicode string version */
typedef struct _ADMXRC3_FPGA_INFOW {
  wchar_t                Identifier[32];    /* Upper case string in form as seen in .BIT file          */
  ADMXRC3_FPGA_TYPE      DeviceCode;        /* Device; see ADMXRC2_FPGA_TYPE                           */
  ADMXRC3_FAMILY_TYPE    FamilyCode;        /* Family; see ADMXRC2_FAMILY_TYPE                         */
  ADMXRC3_SUBFAMILY_TYPE SubfamilyCode;     /* Subfamily; see ADMXRC2_SUBFAMILY_TYPE                   */
  ADMXRC3_PACKAGE_TYPE   PackageCode;       /* Package; see ADMXRC2_PACKAGE_TYPE                       */
  wchar_t                Device[16];        /* As 'DeviceCode', but upper case string e.g. "2VP7"      */
  wchar_t                Package[16];       /* As 'Package', but upper case string e.g. "FG456"        */
  _ADMXRC3_UINT32        Flags;             /* Indicates which of following fields are valid           */
  wchar_t                SpeedGrade[8];     /* Speed grade including "C" or "I"                        */
  wchar_t                Stepping[8];       /* Stepping level, or -1 if not known                      */
  _ADMXRC3_BOOL          Present;           /* TRUE if the device is physically present                */ 
} ADMXRC3_FPGA_INFOW;

/*
** Version information structure, returned by ADMXRC3_GetVersionInfo
*/

typedef struct _ADMXRC3_VERSION_INFO {
  struct _ApiVersion {
    _ADMXRC3_UINT16 Major;
    _ADMXRC3_UINT16 Minor;
    _ADMXRC3_UINT16 Bugfix;
  } Api;
  struct _DriverVersion {
    _ADMXRC3_UINT16 Major;
    _ADMXRC3_UINT16 Minor;
    _ADMXRC3_UINT16 Bugfix;
  } Driver;
} ADMXRC3_VERSION_INFO;

/*
** Local bus window information structure, returned by ADMXRC3_GetWindowInfo
*/

typedef struct _ADMXRC3_WINDOW_INFO {
  _ADMXRC3_UINT64 BusSize;
  _ADMXRC3_UINT64 BusBase;
  _ADMXRC3_UINT64 LocalSize;    /* 0 if no presence on local bus */
  _ADMXRC3_UINT64 LocalBase;
  _ADMXRC3_UINT64 VirtualSize;
} ADMXRC3_WINDOW_INFO;

#endif
